@@ -1,6 +1,6 @@
 //
 //  Little cms
-//  Copyright (C) 1998-2003 Marti Maria
+//  Copyright (C) 1998-2005 Marti Maria
 //
 // Permission is hereby granted, free of charge, to any person obtaining 
 // a copy of this software and associated documentation files (the "Software"), 
@@ -22,7 +22,7 @@
 
 // Test Suite for Little cms
 
-// #define ICM_COMPARATIVE      1
+//#define ICM_COMPARATIVE      1
 //#define CHECK_SPEED          1
 
 #ifdef __BORLANDC__
@@ -30,6 +30,9 @@
 #endif
 
 #include "lcms.h"
+
+
+
 #include <time.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -38,6 +41,7 @@
 #include <icm.h>
 #endif
 
+
 #define PREC  20
 
 #define TYPE_XYZA_16            (COLORSPACE_SH(PT_XYZ)|CHANNELS_SH(3)|BYTES_SH(2)|EXTRA_SH(1))
@@ -45,6 +49,9 @@
 
 typedef struct {BYTE r, g, b, a;}   Scanline_rgb1;
 typedef struct {WORD r, g, b, a;}   Scanline_rgb2;
+typedef struct {BYTE r, g, b;}      Scanline_rgb8;
+typedef struct {WORD r, g, b;}      Scanline_rgb0;
+
 
 // Print a dot for gauging
 
@@ -103,7 +110,7 @@ int CheckSwab(void)
         return 1;
 #endif
 
-    swab(Test, Test, 6);
+    swab((char*) Test, (char*) Test, 6);
 
     if (strncmp((char*) Test, "\x2\x1\x4\x3\x6\x5", 6) != 0)
     {
@@ -115,6 +122,25 @@ int CheckSwab(void)
     return 1;
 }
 
+
+static
+int CheckQuickFloor(void)
+{
+        
+    if ((_cmsQuickFloor(1.234) != 1) ||
+        (_cmsQuickFloor(32767.234) != 32767) ||
+        (_cmsQuickFloor(-1.234) != -2) ||
+        (_cmsQuickFloor(-32767.1) != -32768)) {
+
+                printf("\nOOOPPSS! _cmsFloor() does not work as expected in your machine!\n\n");
+                printf("Please, edit lcms.h and uncomment the LCMS_DEFAULT_FLOOR_CONVERSION toggle.\n");
+                return 0;
+
+    }
+
+    return 1;
+}
+
 typedef struct _Stats {
                       double n, x, y, x2, y2, xy;
                       double Peak;
@@ -307,7 +333,7 @@ static
 int TestLinearInterpolation(int lExhaustive)
 {
        static WORD Tab[4098];
-       int j, i, k;
+       int j, i, k = 0;
        L16PARAMS p;
        int n;
        clock_t time;
@@ -712,9 +738,11 @@ int Test3D(void)
 
        cmsEvalLUT(MyLut, In, Out);
 
-      // Check results, I will tolerate error <= 1  for rounding
 
-       // printf("%x, %x\n", In[0], Out[0]);
+       
+        // Check results, I will tolerate error <= 1  for rounding
+
+       
 
        if (!IsGood("Channel 1", Out[0], In[0])) return 0;
        if (!IsGood("Channel 2", Out[1], (WORD) ((double) In[1] / 2))) return 0;
@@ -722,6 +750,8 @@ int Test3D(void)
 
       }
 
+   
+   
    // Last, remember free stuff
 
    cmsFreeLUT(MyLut);
@@ -1033,7 +1063,7 @@ int CompareTransforms(cmsHTRANSFORM xform1, cmsHTRANSFORM xform2,
                      {
                             // I will test random LSB
 
-                            for (b=0; b < BASE; b++)     // 256
+                            for (b=0; b <= BASE; b++)     // 256
                             {
 
                             bin[b].r = RGB_8_TO_16(r);
@@ -1496,6 +1526,7 @@ int TestDeviceLinkGeneration()
     cmsCloseProfile(hDevLink);
     cmsCloseProfile(hsRGB);
     cmsDeleteTransform(hIdentity);
+	
 
     hDevLink = cmsOpenProfileFromFile("devicelink.icm", "r");
     hXForm   = cmsCreateTransform(hDevLink, TYPE_RGBA_16, NULL, TYPE_RGBA_16, INTENT_RELATIVE_COLORIMETRIC, 0); 
@@ -1538,7 +1569,7 @@ int TestInkLimiting()
             for (j=0; j < 255; j += 8) 
                 for (i=0; i < 255; i += 8) {
 
-                    In[0] = i; In[1] = j; In[2] = k; In[3] = l;
+                    In[0] = (BYTE) i; In[1] = (BYTE) j; In[2] = (BYTE) k; In[3] = (BYTE) l;
 
                     cmsDoTransform(hXForm, In, Out, 1);
 
@@ -1612,13 +1643,13 @@ void CompareWithICM_16bit(void)
     
 
     Profile.dwType = PROFILE_FILENAME;
-    Profile.pProfileData = "sRGBSpac.ICM";
-    Profile.cbDataSize   = strlen("sRGBSpac.ICM");
+    Profile.pProfileData = "sRGBSpac.icm";
+    Profile.cbDataSize   = strlen("sRGBSpac.icm");
 
     hICMProfileFrom = OpenColorProfile(&Profile, PROFILE_READ, FILE_SHARE_READ, OPEN_EXISTING);
 
-    Profile.pProfileData = "sRGBSpac.ICM";
-    Profile.cbDataSize   = strlen("sRGBSpac.ICM");
+    Profile.pProfileData = "sRGBSpac.icm";
+    Profile.cbDataSize   = strlen("sRGBSpac.icm");
     hICMProfileTo   = OpenColorProfile(&Profile, PROFILE_READ, FILE_SHARE_READ, OPEN_EXISTING);
 
     ZeroMemory(&LogColorSpace, sizeof(LOGCOLORSPACE));
@@ -1626,7 +1657,7 @@ void CompareWithICM_16bit(void)
     LogColorSpace.lcsSignature = LCS_SIGNATURE;
     LogColorSpace.lcsVersion   = 0x400;
     LogColorSpace.lcsCSType    = LCS_CALIBRATED_RGB;
-    strcpy(LogColorSpace.lcsFilename, "sRGBSpac.ICM");
+    strcpy(LogColorSpace.lcsFilename, "sRGBSpac.icm");
 
     hICMxform = CreateColorTransform(&LogColorSpace, hICMProfileTo, NULL, BEST_MODE);
 
@@ -1655,8 +1686,8 @@ void CompareWithICM_16bit(void)
     CloseColorProfile(hICMProfileTo);
     DeleteColorTransform(hICMxform);
 
-    hlcmsProfileIn  = cmsOpenProfileFromFile("sRGBSpac.ICM", "r");
-    hlcmsProfileOut = cmsOpenProfileFromFile("sRGBSpac.ICM", "r");
+    hlcmsProfileIn  = cmsOpenProfileFromFile("sRGBSpac.icm", "r");
+    hlcmsProfileOut = cmsOpenProfileFromFile("sRGBSpac.icm", "r");
 
     hlcmsxform  = cmsCreateTransform(hlcmsProfileIn, TYPE_RGB_16, hlcmsProfileOut, TYPE_RGB_16, INTENT_PERCEPTUAL, 0);
 
@@ -1706,13 +1737,13 @@ void CompareWithICM_8bit(void)
     
 
     Profile.dwType = PROFILE_FILENAME;
-    Profile.pProfileData = "sRGBSpac.ICM";
-    Profile.cbDataSize   = strlen("sRGBSpac.ICM");
+    Profile.pProfileData = "sRGBSpac.icm";
+    Profile.cbDataSize   = strlen("sRGBSpac.icm");
 
     hICMProfileFrom = OpenColorProfile(&Profile, PROFILE_READ, FILE_SHARE_READ, OPEN_EXISTING);
 
-    Profile.pProfileData = "sRGBSpac.ICM";
-    Profile.cbDataSize   = strlen("sRGBSpac.ICM");
+    Profile.pProfileData = "sRGBSpac.icm";
+    Profile.cbDataSize   = strlen("sRGBSpac.icm");
     hICMProfileTo   = OpenColorProfile(&Profile, PROFILE_READ, FILE_SHARE_READ, OPEN_EXISTING);
 
     ZeroMemory(&LogColorSpace, sizeof(LOGCOLORSPACE));
@@ -1720,7 +1751,7 @@ void CompareWithICM_8bit(void)
     LogColorSpace.lcsSignature = LCS_SIGNATURE;
     LogColorSpace.lcsVersion   = 0x400;
     LogColorSpace.lcsCSType    = LCS_CALIBRATED_RGB;
-    strcpy(LogColorSpace.lcsFilename, "sRGBSpac.ICM");
+    strcpy(LogColorSpace.lcsFilename, "sRGBSpac.icm");
 
     hICMxform = CreateColorTransform(&LogColorSpace, hICMProfileTo, NULL, BEST_MODE);
 
@@ -1751,8 +1782,8 @@ void CompareWithICM_8bit(void)
     CloseColorProfile(hICMProfileTo);
     DeleteColorTransform(hICMxform);
 
-    hlcmsProfileIn  = cmsOpenProfileFromFile("sRGBSpac.ICM", "r");
-    hlcmsProfileOut = cmsOpenProfileFromFile("sRGBSpac.ICM", "r");
+    hlcmsProfileIn  = cmsOpenProfileFromFile("sRGBSpac.icm", "r");
+    hlcmsProfileOut = cmsOpenProfileFromFile("sRGBSpac.icm", "r");
 
     hlcmsxform  = cmsCreateTransform(hlcmsProfileIn, TYPE_BGRA_8, hlcmsProfileOut, TYPE_BGRA_8, INTENT_PERCEPTUAL, 0);
 
@@ -1788,42 +1819,111 @@ void CompareWithICM_8bit(void)
 
 
 #ifdef CHECK_SPEED
+
+
+
 static
 void SpeedTest(void)
 {
 
-    int r, g, b;
+    int r, g, b, j;
     clock_t atime;
     double seconds, diff;
     cmsHPROFILE hlcmsProfileIn, hlcmsProfileOut;
     cmsHTRANSFORM hlcmsxform;
-    COLOR In, Out;
-   
-   
-    hlcmsProfileIn  = cmsOpenProfileFromFile("sRGB Color Space Profile.ICM", "r");
-    hlcmsProfileOut = cmsOpenProfileFromFile("sRGB Color Space Profile.ICM", "r");
+    Scanline_rgb0 *In;
+	size_t Mb;
+   	
+    hlcmsProfileIn  = cmsOpenProfileFromFile("sRGB Color Space Profile.icm", "r");
+    hlcmsProfileOut = cmsOpenProfileFromFile("sRGBSpac.icm", "r");
+
+    hlcmsxform  = cmsCreateTransform(hlcmsProfileIn, TYPE_RGB_16, hlcmsProfileOut, TYPE_RGB_16, INTENT_PERCEPTUAL, cmsFLAGS_NOTCACHE);
+
+	Mb = 256*256*256*sizeof(Scanline_rgb0);
+
+	In = malloc(Mb);
+
+	j = 0;
+	for (r=0; r < 256; r++)
+        for (g=0; g < 256; g++)
+            for (b=0; b < 256; b++) {
+
+        In[j].r = (r << 8) | r;
+        In[j].g = (g << 8) | g;
+        In[j].b = (b << 8) | b;
+
+		j++;
+	}
 
-    hlcmsxform  = cmsCreateTransform(hlcmsProfileIn, TYPE_RGB_16, hlcmsProfileOut, TYPE_RGB_16, INTENT_PERCEPTUAL, 0);
 
     printf("lcms is transforming full spectrum...");
 
     atime = clock();
 
-    for (r=0; r < 255; r++)
-        for (g=0; g < 255; g++)
-            for (b=0; b < 255; b++) {
-
-        In.rgb.red   = (r << 8) | r;
-        In.rgb.green = (g << 8) | g;
-        In.rgb.blue  = (b << 8) | b;
-
-        cmsDoTransform(hlcmsxform, &In.rgb, &Out.rgb, 1);
-    }
+    cmsDoTransform(hlcmsxform, In, In, 256*256*256);
 
     diff = clock() - atime;
     seconds = (double) diff / CLOCKS_PER_SEC;
+	free(In);
+	
     
-    printf("done.\n[%d tics, %g sec, %g Mb/sec.]\n", (int) diff, seconds, 32. / seconds );
+    printf("done.\n[%d tics, %g sec, %g Mpixel/sec.]\n", (int) diff, seconds, Mb / (1024*1024*seconds*3*2) );
+
+    cmsDeleteTransform(hlcmsxform);
+    cmsCloseProfile(hlcmsProfileIn);
+    cmsCloseProfile(hlcmsProfileOut);
+
+}
+
+
+
+static
+void SpeedTest2(void)
+{
+
+    int r, g, b, j;
+    clock_t atime;
+    double seconds, diff;
+    cmsHPROFILE hlcmsProfileIn, hlcmsProfileOut;
+    cmsHTRANSFORM hlcmsxform;
+    Scanline_rgb8 *In;
+	size_t Mb;
+   
+   
+    hlcmsProfileIn  = cmsOpenProfileFromFile("sRGB Color Space Profile.icm", "r");
+    hlcmsProfileOut = cmsOpenProfileFromFile("sRGBSpac.icm", "r");
+
+    hlcmsxform  = cmsCreateTransform(hlcmsProfileIn, TYPE_RGB_8, hlcmsProfileOut, TYPE_RGB_8, INTENT_PERCEPTUAL, cmsFLAGS_NOTCACHE);
+
+	Mb = 256*256*256*sizeof(Scanline_rgb8);
+
+	In = malloc(Mb);
+
+	j = 0;
+	for (r=0; r < 256; r++)
+        for (g=0; g < 256; g++)
+            for (b=0; b < 256; b++) {
+
+        In[j].r = r;
+        In[j].g = g;
+        In[j].b = b;
+
+		j++;
+	}
+
+
+    printf("lcms is transforming full spectrum...");
+
+    atime = clock();
+
+    cmsDoTransform(hlcmsxform, In, In, 256*256*256);
+
+    diff = clock() - atime;
+    seconds = (double) diff / CLOCKS_PER_SEC;
+	free(In);
+	
+    
+    printf("done.\n[%d tics, %g sec, %g Mpixels/sec.]\n", (int) diff, seconds, Mb / (1024*1024*seconds*3) );
 
     cmsDeleteTransform(hlcmsxform);
     cmsCloseProfile(hlcmsProfileIn);
@@ -1859,7 +1959,7 @@ int TestSaveToMem(void)
     if (_cmsSaveProfileToMem(hProfile, memPtr, &bytesNeeded)) {
             
 
-        cmsHPROFILE newProfile = cmsOpenProfileFromMem(memPtr, bytesNeeded);        
+        cmsHPROFILE newProfile = cmsOpenProfileFromMem(memPtr, (DWORD) bytesNeeded);        
         const char* s = cmsTakeProductName(newProfile);
 
         if (strncmp(s, "sRGB", 4) == 0) rc = TRUE;
@@ -1936,7 +2036,7 @@ int TestNamedColor(void)
         WORD index;
         WORD Color[3];
 
-        index = i;
+        index = (WORD) i;
 
         cmsDoTransform(xform, &index, Color, 1);
 
@@ -1977,6 +2077,43 @@ int TestNamedColor(void)
     return 1;
 }
 
+
+static
+int TestColorantTableTag()
+{
+	LPcmsNAMEDCOLORLIST	     nc2;
+	cmsHPROFILE hProfile = cmsOpenProfileFromFile("colTable.icc", "w");
+
+	nc2 = cmsAllocNamedColorList(3);
+
+	strcpy(nc2 ->List[0].Name, "Red");
+	strcpy(nc2 ->List[1].Name, "Green");
+	strcpy(nc2 ->List[2].Name, "Blue");
+
+	
+    cmsSetDeviceClass(hProfile, icSigOutputClass);
+    cmsSetPCS(hProfile, icSigLabData);
+    cmsSetColorSpace(hProfile, icSigRgbData);
+
+    cmsAddTag(hProfile, icSigColorantTableTag, (void*) nc2);
+    cmsAddTag(hProfile, icSigMediaWhitePointTag, cmsD50_XYZ());
+    cmsCloseProfile(hProfile);
+    cmsFreeNamedColorList(nc2);
+
+
+	hProfile = cmsOpenProfileFromFile("colTable.icc", "r");
+
+	nc2 = cmsReadColorantTable(hProfile, icSigColorantTableTag);
+
+	cmsFreeNamedColorList(nc2);
+	cmsCloseProfile(hProfile);
+
+	unlink("colTable.icc");
+
+	return 1;
+
+}
+
 // New to 1.13 -- CGATS/IT8.7
 
 
@@ -1993,7 +2130,7 @@ int TestIT8(void)
     it8 = cmsIT8Alloc();
 
     cmsIT8SetSheetType(it8, "LCMS/TESTING");
-    cmsIT8SetPropertyStr(it8, "ORIGINATOR",   "1");
+    cmsIT8SetPropertyStr(it8, "ORIGINATOR",   "1 2 3 4");
     cmsIT8SetPropertyUncooked(it8, "DESCRIPTOR",   "1234");
     cmsIT8SetPropertyStr(it8, "MANUFACTURER", "3");
     cmsIT8SetPropertyDbl(it8, "CREATED",      4);
@@ -2023,6 +2160,13 @@ int TestIT8(void)
     cmsIT8SaveToFile(it8, "TEST.IT8");
     cmsIT8Free(it8);
 
+
+	it8 = cmsIT8LoadFromFile("TEST.IT8");
+	cmsIT8SaveToFile(it8, "TEST.IT8");
+	cmsIT8Free(it8);
+
+
+
     it8 = cmsIT8LoadFromFile("TEST.IT8");
 
     if (cmsIT8GetPropertyDbl(it8, "DESCRIPTOR") != 1234) {
@@ -2031,6 +2175,16 @@ int TestIT8(void)
         return 0;
     }
 
+
+	cmsIT8SetPropertyDbl(it8, "DESCRIPTOR", 5678);
+
+	if (cmsIT8GetPropertyDbl(it8, "DESCRIPTOR") != 5678) {
+    
+        printf("fail!\n");
+        return 0;
+    }
+
+
     if (cmsIT8GetDataDbl(it8, "P3", "RGB_G") != 3) {
         printf("fail!\n");
         return 0;
@@ -2046,18 +2200,90 @@ int TestIT8(void)
 }
 
 
+// Create CSA/CRD
+
+static
+void GenerateCSA(const char* cInProf)
+{
+	cmsHPROFILE hProfile;
+	
+	
+	size_t n;
+	char* Buffer;
+
+
+	if (cInProf == NULL) 
+		hProfile = cmsCreateLabProfile(NULL);
+	else 
+		hProfile = cmsOpenProfileFromFile(cInProf, "r");
+
+	n = cmsGetPostScriptCSA(hProfile, 0, NULL, 0);
+	if (n == 0) return;
+
+	Buffer = (char*) malloc(n + 1);
+	cmsGetPostScriptCSA(hProfile, 0, Buffer, n);
+	Buffer[n] = 0;
+	
+	free(Buffer);
+	cmsCloseProfile(hProfile);
+}
+
+
+static
+void GenerateCRD(const char* cOutProf)
+{
+	cmsHPROFILE hProfile;
+	size_t n;
+	char* Buffer;
+    DWORD dwFlags = 0;
+    
+
+	if (cOutProf == NULL) 
+		hProfile = cmsCreateLabProfile(NULL);
+	else 
+		hProfile = cmsOpenProfileFromFile(cOutProf, "r");
+
+	n = cmsGetPostScriptCRDEx(hProfile, 0, dwFlags, NULL, 0);
+	if (n == 0) return;
+
+	Buffer = (char*) malloc(n + 1);
+    cmsGetPostScriptCRDEx(hProfile, 0, dwFlags, Buffer, n);
+	Buffer[n] = 0;
+	
+	free(Buffer);
+	cmsCloseProfile(hProfile);
+}
+
+static 
+int TestPostScript()
+{
+	GenerateCSA("sRGB Color Space Profile.icm");
+	GenerateCRD("sRGB Color Space Profile.icm");
+	GenerateCSA(NULL);
+	GenerateCRD(NULL);
+
+	return 1;
+}
+
+
+
 
 int main(int argc, char *argv[])
 {
        int lExhaustive = 0;   
-
        
+	
+	  // #include "crtdbg.h"
+	  //_CrtSetDbgFlag ( _CRTDBG_ALLOC_MEM_DF | _CRTDBG_LEAK_CHECK_DF ); 
+
+	  	   
        printf("little cms testbed. Ver %1.2f [build %s %s]\n\n", LCMS_VERSION / 100., __DATE__, __TIME__);
        
 #ifndef LCMS_DLL
 
-       if (!CheckEndianess()) return 1;
+       if (!CheckEndianess()) return 1;	   
        if (!CheckSwab()) return 1;
+	   if (!CheckQuickFloor()) return 1;
 
        TestFixedPoint();
      
@@ -2067,28 +2293,37 @@ int main(int argc, char *argv[])
        if (!TestLinearInterpolation(lExhaustive)) return 1;
        if (!TestReverseLinearInterpolation()) return 1;
 
+	   
        
 
        if (!Test3D()) return 1;
        if (!TestMatrixCreation()) return 1;
        if (!GetInfoTest()) return 1;
 
+
 #endif
 
        if (!Test_sRGB()) return 1;
+	
        if (!RealProfilesTest()) return 1;
-       if (!TestInducedError(TYPE_LABA_16)) return 1;
-
+       if (!TestInducedError(TYPE_LABA_16)) return 1;	   
        if (!TestPreview()) return 1;
-       if (!TestMultiprofile()) return 1;
+       if (!TestMultiprofile()) return 1; 	   	 
        if (!TestLinearizationDevicelink()) return 1;
-       if (!TestDeviceLinkGeneration()) return 1;
+	  	 
+       if (!TestDeviceLinkGeneration()) return 1; 	  
+	   
+
        if (!TestLinearizationDevicelink2()) return 1;
+	  
+
        if (!TestInkLimiting()) return 1;
        if (!TestSaveToMem()) return 1;
        if (!TestNamedColor()) return 1;  
        if (!TestIT8()) return 1;
-
+	   if (!TestPostScript()) return 1;
+	   if (!TestColorantTableTag()) return 1;
+	   
 #ifdef ICM_COMPARATIVE
 #ifndef NON_WINDOWS
        CompareWithICM_8bit();
@@ -2098,6 +2333,7 @@ int main(int argc, char *argv[])
 
 #ifdef CHECK_SPEED    
        SpeedTest();
+	   SpeedTest2();
 #endif
 
        printf("\nSuccess.\n");
@@ -2107,3 +2343,8 @@ int main(int argc, char *argv[])
 }
 
 
+
+
+
+
+
